package com.eco.plugin.xxx.zgdxsso.filter;

import com.eco.plugin.xxx.zgdxsso.config.PluginSimpleConfig;
import com.eco.plugin.xxx.zgdxsso.db.bean.DBEntity;
import com.eco.plugin.xxx.zgdxsso.db.controller.DBController;
import com.eco.plugin.xxx.zgdxsso.utils.EncryptUtils;
import com.eco.plugin.xxx.zgdxsso.utils.FRUtils;
import com.eco.plugin.xxx.zgdxsso.utils.ResponseUtils;
import com.eco.plugin.xxx.zgdxsso.utils.Utils;
import com.fr.decision.fun.impl.AbstractGlobalRequestFilterProvider;
import com.fr.json.JSONObject;
import com.fr.plugin.context.PluginContexts;
import com.fr.record.analyzer.EnableMetrics;
import com.fr.stable.fun.Authorize;
import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.HashMap;
import java.util.Map;

@EnableMetrics
@Authorize(callSignKey = "com.eco.plugin.xxx.zgdxsso")
public class SSOFilter extends AbstractGlobalRequestFilterProvider {
    @Override
    public String filterName() {
        return "zgdxssoFilter";
    }

    @Override
    public String[] urlPatterns() {
        return new String[]{"/*"};
    }

    @Override
    public void doFilter(HttpServletRequest req, HttpServletResponse res, FilterChain chain ){

        if(PluginContexts.currentContext().isAvailable()){
            Long now = System.currentTimeMillis();
            String token = req.getParameter("ssoTokenStr");
            //如果token为空，则放行
            if(Utils.isNullStr(token)){
                release(req,res,chain);
                return;
            }

            String uuid = req.getParameter("keyId");
            DBEntity keyRecord = DBController.getById(uuid);

            String privateKey = keyRecord.getPrivateKey();
            String used = keyRecord.getUsed();

            if(keyRecord == null || Utils.isNullStr(privateKey)||used.equals("Y")){
                ResponseUtils.responseText(res,"token已失效，请重新登陆！");
                return ;
            }

            JSONObject info = null;
            try {
                info = decrypt(token,privateKey);
            } catch (Exception e) {
                ResponseUtils.responseText(res,"token校验失败，请联系管理员！");
                return ;
            }
            String username = info.getString("username");
            Long timestamp = info.getLong("timestamp");
            String timeout = PluginSimpleConfig.getInstance().getTimeout();

            if(Utils.isNotNullStr(timeout)){
                int timeoutint = Integer.parseInt(timeout);

                if((now - timestamp) / 1000 > timeoutint){
                    ResponseUtils.responseText(res,"token已失效，请重新登陆！");
                    return ;
                }
            }

            //将记录修改为已使用状态
            keyRecord.setUsed("Y");
            DBController.single(keyRecord,null);

            String url = Utils.removeParam(FRUtils.getAllUrl(req),"ssoTokenStr");
            FRUtils.login(req,res,username,url);
        }

        release(req,res,chain);
    }

    /**
     * 解密Token
     * @param token
     * @param privateKey
     * @return
     */
    private JSONObject decrypt(String token, String privateKey) throws Exception {
        return new JSONObject(EncryptUtils.rsaDecrypt(token,privateKey));
    }

    //放行拦截器
    private void release(HttpServletRequest req, HttpServletResponse res, FilterChain chain) {
        try{
            chain.doFilter(req,res);
        }catch (Exception e){
            FRUtils.FRLogInfo("拦截失败");
        }
    }
}

